//============================================================================
// MapVoteClientIntro
//============================================================================
//
class MapVoteClientIntro extends Mutator config(MapVoteXL);
//
#exec texture IMPORT NAME=mvxlogo File=Textures\mvxlogo.pcx GROUP=MapVoteX MIPS=OFF

var MapVoteX MyMut;

struct LogoNames
{
	var bool   bTransparent;
	var string ServerLogoName;
	var string LogoSoundName;
	var int    X, Y;
};

var() config bool bUseOnlySplashLogo_0;
var() config LogoNames SplashLogo[30];

var bool bInitialized;
var bool bNewClient;
var bool bSoundDone;
var bool bEnableLogo;
var int  GameStart;
var int  GameNum;

var ChallengeHUD MyHUD;
var Texture mvx1, ServerLogo;
var Sound MySound;
var int DrawLogo, LogoCounter;
var Color WhiteColor;
var PlayerPawn PlayerOwner;

var int TickTime;

var bool bDone;
var bool bClear;
var string LogoName, SoundName;
var int Xpos, Ypos;

replication
{
   reliable if (Role == ROLE_Authority)
               bClear, LogoName, SoundName, Xpos, Ypos, GameStart, GameNum, bEnableLogo;
}

simulated function PostBeginPlay()
{
    Super.PostBeginPlay();

    if ( !bInitialized )
    {
       bInitialized = true;
       GameNum = 0;
       SetTimer(1, true);
       SaveConfig();
    }
}

simulated function Tick( float delta )
{
    local int i;

    if (Level.NetMode != NM_Client && GameNum > 0 && !bDone)
    {
        GameNum -= 1;
        if (bUseOnlySplashLogo_0)
            GameNum = 0;
        bClear    = SplashLogo[GameNum].bTransParent;
        LogoName  = SplashLogo[GameNum].ServerLogoName;
        SoundName = SplashLogo[GameNum].LogoSoundName;
        Xpos      = SplashLogo[GameNum].X;
        Ypos      = SplashLogo[GameNum].Y;
        bDone     = true;
    }

    if ( Level.NetMode != NM_DedicatedServer )
    {
       if ( !bHUDMutator )
          RegisterHUDMutator();

       if ( PlayerOwner != None )
       {
          if (!bNewClient)
          {
             if (TickTime++ > 50)
             {
                bNewClient = True;
                if (!Level.Game.bGameEnded)
                {
                   DrawLogo = 1;
                   if (GameStart == 1)
                       SetTimer(0.02, true);
                   else
                       SetTimer(0.08 , True);
                }
             }
          }
       }
    }
}

simulated function Timer()
{
    Super.Timer();

    if ( Level.NetMode != NM_DedicatedServer )
    {
       if ( DrawLogo > 0 )
       {
          LogoCounter++;
          if ( DrawLogo == 510 )
          {
             DrawLogo = 0;
             SetTimer( 0.0, False );
          }
          else
          if ( LogoCounter > 20 )
          {
             DrawLogo += 8;
             if ( DrawLogo > 510 )
                DrawLogo = 510;
          }
          else
          if ( LogoCounter == 20 )
          {
             DrawLogo = 5;
          }
       }
    }
}

simulated event PostRender( Canvas C )
{
    local float Xposy, Yposy;
    local int   Xp, Yp, Zpos;

    if ( PlayerOwner == none )
    {
       PlayerOwner = C.Viewport.Actor;
       MyHUD = ChallengeHUD( PlayerOwner.MyHUD );
    }

    C.Style = ERenderStyle.STY_Translucent;

    if ( DrawLogo != 0 )
    {
       if ( DrawLogo > 1 )
       {
          C.DrawColor.R = 255 - DrawLogo/2;
          C.DrawColor.G = 255 - DrawLogo/2;
          C.DrawColor.B = 255 - DrawLogo/2;
       }
       else
       {
          C.DrawColor = WhiteColor;
       }

       if (mvx1 == None)
      	   mvx1 = texture'mvxlogo';
       Xposy = C.ClipX - (mvx1.Usize * 0.75);
       C.SetPos( Xposy, C.ClipY - (mvx1.Usize * 0.75));
       C.DrawIcon( mvx1, 0.65 );

       if (bEnableLogo)
       {
           if (ServerLogo == None && InStr(LogoName, ".") > 0)
           {
               ServerLogo = Texture(DynamicLoadObject(LogoName, class'Texture'));
               if (MySound == None && InStr(SoundName, ".") > 0)
                   MySound = Sound(DynamicLoadObject(SoundName, class'Sound'));
           }

           if (Serverlogo != None)
           {
               Xp = Xpos;
               if (Xp <= 0)
                   Xp = 50;
               Yp = Ypos;
               if (Yp <= 0)
                   Yp = 70;
               Xposy = (C.ClipX * Xp / 100) - (ServerLogo.Usize / 2);
               Yposy = (C.ClipY * Yp / 100) - (ServerLogo.Usize / 2);
               C.SetPos( Xposy, Yposy);
               if (!bClear)
                   C.Style = ERenderStyle.STY_Normal;
               C.DrawIcon( ServerLogo, 1.0 );
               if (!bSoundDone && MySound != None)
               {
                   bSoundDone = true;
                   PlayerOwner.PlaySound(MySound);
               }
           }
       }
    }

    C.Style = ERenderStyle.STY_Normal;

    if (NextHUDMutator != None)
        NextHUDMutator.PostRender(C);
}
//
defaultproperties
{
    WhiteColor=(R=255,G=255,B=255)
    bAlwaysRelevant=True
    RemoteRole=ROLE_SimulatedProxy
}
