//=================================================================
// MapVoteCommands
//=================================================================
//
class MapVoteCommands extends Mutator;
//
var bool bInitialized;

function PostBeginPlay()
{
    if (!bInitialized)
    {
       bInitialized = true;
       Level.Game.BaseMutator.AddMutator(Self);
       Level.Game.RegisterMessageMutator(Self);
    }
    Super.PostBeginPlay();
}

function bool MutatorBroadcastMessage( Actor Sender, Pawn Receiver, out coerce string Msg, optional bool bBeep, out optional name Type )
{
	if (Sender == Receiver && Sender.IsA('Spectator'))
	{
		if (CheckMessage(Mid(Msg,InStr(Msg,":")+1), Receiver))
		   return false;
	}
	return Super.MutatorBroadcastMessage(Sender,Receiver,Msg,bBeep,Type);
}

function bool MutatorTeamMessage(Actor Sender, Pawn Receiver, PlayerReplicationInfo PRI, coerce string Msg, name Type, optional bool bBeep)
{
	if (Sender == Receiver)
	{
		if (CheckMessage(Msg, Receiver))
		   return false;
	}
	return Super.MutatorTeamMessage(Sender,Receiver,PRI,Msg,Type,bBeep);
}

function bool CheckMessage(String Msg, Pawn Sender)
{
    local bool bSwitch;

    if (TeamGamePlus(Level.Game)!=None && !TeamGamePlus(Level.Game).bNoTeamChanges)
        bSwitch = true;

	if (Sender.IsA('PlayerPawn') && !Sender.IsA('Spectator') && bSwitch)
	{
		if (Msg ~= "!RED" || Msg ~= "!R")
		{
			ChangePlayerToTeam(PlayerPawn(Sender),0,false);
			return true;
		}
		if (Msg ~= "!BLUE" || Msg ~= "!B")
		{
			ChangePlayerToTeam(PlayerPawn(Sender),1,false);
			return true;
		}
	}

	if (Sender.IsA('PlayerPawn') && !Sender.IsA('Spectator'))
	{
		if (Msg ~= "!SPEC" || Msg ~= "!SPECTATE" || Msg ~= "!S")
		{
			PlayerPawn(Sender).ClientTravel("?OverrideClass=Botpack.CHSpectator",TRAVEL_Relative, False);
			return true;
		}
		else
		if (Msg ~= "!U")
		{
		    PlayerPawn(Sender).ConsoleCommand("Suicide");
		    return true;
		}
		else
		if (Msg ~= "!D")
		{
		    PlayerPawn(Sender).ConsoleCommand("FeignDeath");
		    return true;
		}
	}

	if (Sender.IsA('Spectator'))
	{
		if (Msg ~= "!PLAY" || Msg ~= "!P")
		{
			PlayerPawn(Sender).ClientTravel("?OverrideClass=",TRAVEL_Relative, False);
			return true;
		}
	}

    if (!Sender.IsA('Spectator'))
    {
	   if (Msg ~= "!VOTE" || Msg ~= "!MAPVOTE" || Msg ~= "!V")
	   {
		   Level.Game.BaseMutator.Mutate("BDBMAPVOTE VOTEMENU",PlayerPawn(Sender));
		   return true;
	   }
    }

    if (Sender.IsA('PlayerPawn') && Level.NetMode != NM_StandAlone)
    {
	   if (Msg ~= "!SDM" || Msg ~= "!SM")
	   {
		   Level.Game.BaseMutator.Mutate("SMARTDM MENU",PlayerPawn(Sender));
		   return true;
	   }

	   if (Msg ~= "!IPC")
	   {
		   Level.Game.BaseMutator.Mutate("IpToCountry",PlayerPawn(Sender));
		   return true;
	   }
	}

	return false;
}

function ChangePlayerToTeam(Pawn P, int teamnum, bool bInform)
{
	local bool oldbNoTeamChanges;

	if (P.IsA('Spectator') || teamnum == P.PlayerReplicationInfo.Team)
	{
		return;
	}

	if (teamnum < 0 || (TeamGamePlus(Level.Game)!= None && teamnum >= TeamGamePlus(Level.Game).MaxTeams))
	{
		return;
	}

	if (P.IsA('Bot'))
	{
		Bot(P).ConsoleCommand("taunt wave");
	}

	if (TeamGamePlus(Level.Game) != None)
	{
		oldbNoTeamChanges = TeamGamePlus(Level.Game).bNoTeamChanges;
		TeamGamePlus(Level.Game).bNoTeamChanges = False;
	}

	Level.Game.ChangeTeam(P, teamnum);

	if (TeamGamePlus(Level.Game) != None)
	{
		TeamGamePlus(Level.Game).bNoTeamChanges = oldbNoTeamChanges;
	}

	P.Died(None, '', P.Location);

	if (!DeathMatchPlus(Level.Game).bTournament)
	{
		P.PlayerReplicationInfo.Score += 1.0;
	}
}
//
defaultproperties
{
}
