//==============================================================================
// MapVote Windows ReplicationInfo.
//==============================================================================
//
class MapVoteWRI extends ReplicationInfo;
//
var MapVoteX MyMut;

var() Class<UWindowWindow> WindowClass;
var() int  WinLeft;
var() int  WinTop;
var() int  WinWidth;
var() int  WinHeight;
var() bool DestroyOnClose;

var UWindowWindow TheWindow;

var string MyData[15];      // 1=sCode,2=Version,3=Stuff,4=ServerName,5=ServerPort,6=ServerIP,7=Stretch,
                            // 8=Logoname,9=Soundname,10=WebsiteURL,11=ContactURL,12=AnonName,13=AnonURL,
                            // 14=mapPrefix
var int    MyBool[10];      // 1=badmin,2=bgameend,3=bwelcome,4=bwelcomefirst,5=bMaintenanceMode

var string PlayerName[32];
var string MapVoteResults[32];
var bool   bDestroyRequested;

var int    TicksPassed;
var int    gPlayerCount;
var int    gTitleChecksum;

var bool   bDebugMode;
var bool   bUpdated;
var bool   blastCheck;
var bool   bOpenWindowDispatched;
var bool   bSetupWindowDelayDone;

var bool   bAdminMesg;

replication
{
   reliable if ( Role == ROLE_Authority && bAdminMesg)
        bAdminMesg;

   reliable if ( Role == ROLE_Authority && !bAdminMesg )
		MyData, MyBool, PlayerName, MapVoteResults, UpdateMapVoteResults;

   reliable if ( Role == ROLE_Authority )
        OpenWindow, CloseWindow;

   reliable if ( Role < ROLE_Authority )
        DestroyWRI;
}

event PostBeginPlay()
{
	local int i;

	Super.PostBeginPlay();

    for (i=0; i<15; i++)
    {
         MyData[i] = "";
         if (i < 10)
             MyBool[i] = 0;
	}
	OpenIfNecessary();
}

simulated event PostNetBeginPlay()
{
	PostBeginPlay();
	OpenIfNecessary();
}

simulated function OpenIfNecessary()
{
	local PlayerPawn P;

	if ( Owner != None )
	{
		P=PlayerPawn(Owner);
		if ( (P != None) && (P.Player != None) && (P.Player.Console != None) )
		{
			OpenWindow();
		}
	}
}

simulated function bool OpenWindow ()
{
	local PlayerPawn P;
	local WindowConsole C;

	P = PlayerPawn(Owner);
	if ( P == None )
	{
		Log("#### -- Attempted to open a window on something other than a PlayerPawn");
		DestroyWRI();
		return False;
	}

	if ( P.Player.Console.bTyping )
	{
		SetTimer(0.50,False);
		return False;
	}

	bOpenWindowDispatched = True;

	C = WindowConsole(P.Player.Console);
	if ( C == None )
	{
		Log("#### -- No Console");
		DestroyWRI();
		return False;
	}

	if ( !C.bCreatedRoot || (C.Root == None) )
	{
		C.CreateRootWindow(None);
	}
	C.bQuickKeyEnable = True;
	C.LaunchUWindow();
	TicksPassed = 1;
	return True;
}

simulated function Tick (float DeltaTime)
{
	if ( TicksPassed != 0 )
	{
		if ( TicksPassed++  == 5 )
		{
			SetupWindow();
			TicksPassed=0;
		}
	}
	if ( DestroyOnClose && (TheWindow != None) &&  !TheWindow.bWindowVisible &&  !bDestroyRequested )
	{
		bDestroyRequested=True;
		DestroyWRI();
	}
}

simulated function bool SetupWindow()
{
	local WindowConsole C;
	local int i;

	if ( !bSetupWindowDelayDone )
	{
		SetTimer(1.00,False);
		return False;
	}

	DebugLog("MapVoteWRI: SetupWindow()");

	if ( SetWindow() )
	{
		SetTimer(1.00,False);
	}

	else
	{
		Log("Super.SetupWindow() = false");
	}
}

simulated function bool SetWindow()
{
	local WindowConsole C;

	C = WindowConsole(PlayerPawn(Owner).Player.Console);
	TheWindow = C.Root.CreateWindow(WindowClass,WinLeft,WinTop,WinWidth,WinHeight);
	if ( TheWindow == None )
	{
		Log("#### -- CreateWindow Failed");
		DestroyWRI();
		return False;
	}
	if ( C.bShowConsole )
	{
		C.HideConsole();
	}
	TheWindow.bLeaveOnscreen=True;
	TheWindow.ShowWindow();
	return True;
}

simulated function Timer ()
{
	local int i, j, k;
	local int TitleChecksum;
	local int repPlayerCount;
	local string sTemp, GameMap;
	local int List;
	local Color ColColor;

  	ColColor.R=255;
	ColColor.G=255;
	ColColor.B=10;

    if (bAdminMesg)
        return;

	if ( !bOpenWindowDispatched )
	{
		OpenWindow();
		return;
	}

	if ( !bSetupWindowDelayDone )
	{
		bSetupWindowDelayDone=True;
		SetupWindow();                                                                                                     //prefix
	    MapVoteTabWindow(TheWindow).SetupMapWindow(false, MyBool[1], MyBool[3], MyBool[2], MyBool[4], MyBool[5], MyBool[6], MyData[14]);
	}

	bUpdated=False;
                                        //scode      version
	MapVoteTabWindow(TheWindow).SetupMenu(MyData[1], MyData[2], MyBool[1], MyData[3], MyData[4], MyData[5], MyData[6]);
	if (MyBool[3] == 1) // bWelcome
	    MapVoteTabWindow(TheWindow).SetupWelcome(MyData[7], MyData[8], MyData[9], MyData[10], MyData[11], MyData[12], MyData[13]);

    // if ( bAdmin || bGameEnd || (!bGameEnd && bMidGame) )
	if ( MyBool[1] == 1 || MyBool[2] == 1 || (MyBool[2] == 0 && MyBool[6] == 1) )
	{
        for (i=0; i<32; i++)
        {
            if (PlayerName[i] != "")
               repPlayerCount = i + 1;
        }

		if ( repPlayerCount != gPlayerCount )
		{
			for (i=gPlayerCount; i<repPlayerCount; i++)
			{
				MapVoteTabWindow(TheWindow).AddPlayerName(PlayerName[i]); // has voted
			}
		    gPlayerCount = repPlayerCount;
		    bUpdated=True;
		}

        for (i=0; i<31; i++)
        {
            if (MapVoteResults[i] != "")
               UpdateMapVoteResults(MapVoteResults[i], i);
        }

    	if ( TitleChecksum != gTitleChecksum )
    	{
			gTitleChecksum = TitleChecksum;
			bUpdated=True;
		}

		if ( !bUpdated )
    	{
			blastCheck=True;
			SetTimer(1.00,False);
			return;
		}
	}
	blastCheck=False;
	SetTimer(0.50,False);
}

simulated function UpdateMapVoteResults (string Text, int i)
{
	if ( TheWindow != None )
	{
		MapVoteTabWindow(TheWindow).UpdateMapVoteResults(Text,i);
	}
}

simulated function CloseWindow ()
{
	local WindowConsole C;

    if (PlayerPawn(Owner) != None)     // solves access none's if player leaves
    {
	   C = WindowConsole(PlayerPawn(Owner).Player.Console);
	   C.bQuickKeyEnable = False;
    }

	if ( TheWindow != None )
	{
		TheWindow.Close();
	}
}

function DestroyWRI ()
{
	Destroy();
}

function DebugLog (string Msg)
{
	if (bDebugMode)
	    Log("**** MVWRI " $ Msg);
}
//
defaultproperties
{
    WindowClass=Class'MapVoteTabWindow'
    WinWidth=400
    WinHeight=340
    DestroyOnClose=True
    bAlwaysRelevant=False
    RemoteRole=ROLE_SimulatedProxy
    NetPriority=2.00
}