//============================================================================
// MapVoteX - by ProAsm - 2013
//============================================================================
class MapVoteX extends Mutator Config(MapVoteX);
//

// 101
// Fixed initial game map loading if sorted
// Added set GetAllPaths() for maplist + message
// Added bEmptyServerSwitchMap=False
// Corrected mvx logo in admin window
// Cleared vote Gamelist if only 1 game
// 102a
// Added bMajorityVote to midgame as well
// Seperated RepeatLimits
// Added 1st letter map search
// 103
// 102a renamed
//

struct CustConfig
{
	var bool   bUse;
	var string MapFolder;
	var string DefaultMap;
	var string GameName;
	var string GameClass;
	var string Mutators;
	var string Settings;
};

var() config bool bAutoOpen;
var() config bool bAllowMidGameVoting;
var() config bool bDebugMode;
var() config bool bUseCommandOptions;
var() config bool bMajorityPlayerVote;

var() config bool bEnableSplashLogo;
var() config bool bEnableWelcomeMenu;
var() config bool bShowWelcomeFirst;
var() config bool bSortCustomGameList;
var() config bool bEnableEmptyServerSwitch;

var() config int  EmptyServerSwitchTimeMins;
var() config int  GameStartVoteDelay;
var() config int  VoteTimeLimit;
var() config int  ScoreBoardDelay;
var() config int  RepeatLimit;

var() config bool bEmptyServerSwitchMap;
var() config bool bUseCustomGameMapLists;
var() config bool bEnableServerPackages;

var() config int    DefaultCustomGameNum;
var() config string ServerGamePassword;
var() config string ServerIpAddress;
var() config string MaintenanceServerName;
var() config string MaintenancePassword;
var() config string DefaultMutators;
var() config string DefaultSettings;

var() config CustConfig CustomGame[30];

var() config string MVGameName;
var() config string MVTempName;
var() config bool   bMaintenanceMode;

var string MapList[550];
var string GameList[31];
var string Version;
var int CustomList;
var string GameMaps[31];
var int MapCount;
var int GameCount;

var int PlayerIDList[32];
var int PlayerVote[32];
var string MapStatusText[32];

var float EndGameTime;
var bool bMidGameVote;
var int CurrentID;
var int CurrentPlayerCount;
var int JoinTickCount;
var string TravelMap;
var int TimeLeft;
var int ScoreBoardTime;
var int ServerTravelTime;
var Class<GameInfo> CustomGameClass;
var bool bLevelSwitchPending;
var string ServerTravelString;
var bool bInitMapsLoaded;
var string GamePackage;
var string NewPrefix;
var int pos;
var int m, n;

var string LoadedGameClass;
var string LoadedGameName;
var string LoadedMapName;
var string LoadedMapPrefix;
var int    LoadedGameNum;

var bool bInitialized1;
var bool bInitialized2;

var bool bGameHasEnded;
var bool bMidGameTimeout;
var bool bDisplayAdminMessages;

var bool bClassesSpawned;
var bool bRestarting;
var bool bMapsLoaded;
var bool bCheckRules;

var string Servername;
var string ServerPort;
var string ServerIP;
var string sCode;

var MapVoteRepeatLimits    MVRP;
var MapVoteClientIntro     MVCI;
var MapVoteServerPackages  MVSP;
var MapVoteServerActors    MVSA;
var MapVoteServerNames     MVSN;
var MapVoteClientWelcome   MVCW;
var MapVoteAdminMessages   MVAM;
var MapVoteCrashedMaps     MVCM;
var MapVoteGRI             MVGRI;
var MapVoteMapLoader       MVML;

var string GameNames[31];

simulated function PreBeginPlay ()
{
	local int x, j, k;
	local string sFinal, sTemp, sPath;
	local MapVoteCommands MVCMD;
	local MapVoteDefaultSwitch MVDS;
	local CustConfig CusGame;

	if ( !bInitialized1 )
	{
		for (x=0; x<32; x++)
		{
			PlayerIDList[x] = -1;
		}

        CustomList = 30;

		CheckForGameName(MVGameName);
        LoadedGameClass = string(Level.Game.Class);
        LoadedGameName = Level.Game.GameName;
        GetCurrentMap();
        LoadedMapPrefix = GetMapPrefix(LoadedMapName);

        //if (!bUseCustomGameMapLists && InStr(Caps(LoadedMapName), "TUTORIAL") == -1)
        //if (!bUseCustomGameMapLists)
        //    SetMapPath(CustomGame[LoadedGameNum].MapFolder);

		if (!StartMutatorsAndSettings())
 		   return;                      // game restarted

		if (bMaintenanceMode)
		{
		    ExecuteSetting("MinPlayers=2", True);
		    Log("<--------------  Starting MapVoteX in Maintenance Mode  -------------->");
        }
        else
        {
            Log("<------------------------  Starting MapVoteX  ------------------------>");
            Level.ConsoleCommand("set engine.gameinfo GamePassword "$ServerGamePassword);
        }

        if (bUseCommandOptions)
        {
           MVCMD = Spawn(class'MapVoteCommands');
        }

        SaveServername(LoadedGameNum);

		CustomGameClass = None;
		SaveConfig();

        MVRP = Spawn(class'MapVoteRepeatLimits');

        MVCI  = Spawn(class'MapVoteClientIntro');
        MVCI.GameNum = LoadedGameNum + 1;  // so it's > 0 will be made -1
        MVCI.bEnableLogo = bEnableSplashLogo;
        MVCI.MyMut = self;

        if (bEnableWelcomeMenu)
        {
           MVCW = Spawn(class'MapVoteClientWelcome');
           MVCW.MyMut = self;
        }

        if (bEnableEmptyServerSwitch)
        {
           MVDS = Spawn(class'MapVoteDefaultSwitch');
           MVDS.MyMut = self;
           MVDS.ServerSwitchMins = EmptyServerSwitchTimeMins;
        }
		bInitialized1 = True;
	}
	Super.PreBeginPlay();
}

function PostBeginPlay ()
{
    local int i;

    Super.PostBeginPlay();

    if ( !bInitialized2 )
    {
       if (MVRP != None)
       {
           MVRP.RepeatLimit = RepeatLimit;
           MVRP.AdjustMapRepeats(LoadedGameNum);
           MVRP.MyMut = self;
       }

       i = Rand(900);
       if (i < 100)
          i += 100;
       sCode = string(i);
       GetServerInfo();
       bInitialized2 = true;
    }
}

function GetServerInfo()
{
    local int i;

    if (!bMaintenanceMode)
    {
       ServerIP = ServerIpAddress;
       if (Len(ServerIP) < 7)
       {
           ServerIP = Level.GetAddressURL();  // ServerIP = 0.0.0.0:7777 | Port = 7777
       }
       if (ServerIP == "" || Left(ServerIP, 1) == "0")
           return;

       i = InStr(ServerIP, ":");
       if (i >= 0)
       {
           ServerPort = Mid(ServerIP, i+1);
           ServerIP = Left(ServerIP, i);
           i = int(ServerPort);
           i++;
           ServerPort = string(i);
       }
       else
       {
           i = Level.Game.GetServerPort();    // 7777
           i += 1;                            // 7778
           ServerPort = string(i);
       }
       ServerName = class'Engine.GameReplicationInfo'.default.ServerName;
    }
}

function bool HandleEndGame ()
{
	local Pawn aPawn;

	Super.HandleEndGame();

	if (!bAutoOpen || CheckForTie())
	{
		return False;
	}
	if (Level.Game.IsA('Assault'))
	{
		if (  !Assault(Level.Game).bDefenseSet )
		{
			return False;
		}
	}

	DeathMatchPlus(Level.Game).bDontRestart = True;
	TimeLeft = VoteTimeLimit;
	ScoreBoardTime = ScoreBoardDelay;
	bMidGameTimeout = false;
	bMidGameVote = false;
	CloseAllVoteWindows();
	ClearAllVoting();
	SetTimer(1.00,True);
	return False;
}

function GetCurrentMap()
{
   local int i;
   local string sTemp;

   i = InStr(Caps(Level.Game), ".");

   if (i != -1)
   {
      LoadedMapName = Left(Level.Game, i);
   }
   else
   {
      i = InStr(Caps(Level.Game.BaseMutator), ".");
      if (i != -1)
      {
         LoadedMapName = Left(Level.Game.BaseMutator, i);
      }
   }
}

function CheckForGameName(string sName)
{
    local int i;

    LoadedGameNum = -1;

    for (i=0; i<CustomList; i++)
    {
        if (CustomGame[i].bUse && CustomGame[i].GameName == sName)
        {
           LoadedGameNum = i;
           if (CustomGame[i].GameClass == string(Level.Game.Class))
              Level.Game.GameName = CustomGame[i].GameName;
           break;
        }
    }
    for (i=0; i<CustomList; i++) // used in maplist and serveractorpackages
    {
        default.GameNames[i] = CustomGame[i].GameName;
    }
}

event Timer ()
{
	local Pawn aPawn;
	local int VoterNum;
	local int NoVoteCount;
	local int MapNum;
	local Pawn P;
	local int i;

	if ( bLevelSwitchPending )
	{
		if ( Level.TimeSeconds > ServerTravelTime )
		{
			if ( Level.NextURL == "" )
			{
				if ( Level.NextSwitchCountdown < 0 )
				{
					HandleFailedMap(TravelMap);
					MapNum = GetRandomMap();
					ServerTravelString = SetupGameMap(MapList[MapNum]);
					Level.ServerTravel(ServerTravelString,False);
				}
				else
				{
					Level.ServerTravel(ServerTravelString,False);
				}
			}
		}
		return;
	}

	if ( ScoreBoardTime > 0 )
	{
		ScoreBoardTime--;
		if ( ScoreBoardTime == 0 )
		{
			EndGameTime = Level.TimeSeconds;

			for ( aPawn = Level.PawnList; aPawn != None; aPawn = aPawn.nextPawn )
			{
				if ( (PlayerPawn(aPawn) != None) && aPawn.bIsPlayer )
				{
					VoterNum=FindPlayerIndex(PlayerPawn(aPawn).PlayerReplicationInfo.PlayerID,True);
					if ( VoterNum > -1 )
					{
						if ( PlayerVote[VoterNum] == 0 )
						{
							OpenVoteWindow(PlayerPawn(aPawn),False,False);
						}
					}
				}
			}
			BroadcastMessage(string(TimeLeft) $ " seconds left to vote.",True);
		}
		return;
	}

    if (TimeLeft >= 0)
	    TimeLeft--;

	if ( TimeLeft == 60 )
	{
		BroadcastMessage("1 Minute left to vote.",True);

		for ( P=Level.PawnList; P != None; P=P.nextPawn )
		{
			if ( P.IsA('TournamentPlayer') )
			{
				TournamentPlayer(P).TimeMessage(12);
			}
		}
	}
	if ( TimeLeft == 10 )
	{
		BroadcastMessage("10 seconds left to vote.",True);
	}
	if ((TimeLeft < 11) && (TimeLeft > 0))
	{
		if (Level.Game.bGameEnded)
		{
		   for (P=Level.PawnList; P != None; P=P.nextPawn)
		   {
			   if (P.IsA('TournamentPlayer'))
			   {
				   TournamentPlayer(P).TimeMessage(TimeLeft);
			   }
		   }
		}
		else
		{
		   if (TimeLeft < 2)
		   {
		       bMidGameTimeout = true;
		   }
		}
	}
	CleanUpPlayerIDs();

    if (TimeLeft % 20 == 0 && TimeLeft > 0)
    {
        NoVoteCount = 0;
        // force all players voting windows open if they have not voted
        for ( aPawn=Level.PawnList; aPawn!=None; aPawn=aPawn.NextPawn )
        {
            if (aPawn.bIsPlayer && PlayerPawn(aPawn) != none)
            {
                VoterNum = FindPlayerIndex(PlayerPawn(aPawn).PlayerReplicationInfo.PlayerID, True);
                if (PlayerVote[VoterNum] == 0) // if this player has not voted
                {
                    NoVoteCount++;
                    //OpenVoteWindow(PlayerPawn(aPawn));
                }
            }
        }

		if ( NoVoteCount == 0 )
		{
			TallyVotes(True);
		}
	}
	if ( TimeLeft == 0 )
	{
		TallyVotes(True);
	}
}

function Tick (float DeltaTime)
{
	local string PlayerName;
	local string sTemp;
	local string PID;
	local int TeamID;
	local Pawn Other;
	local int PlayerIndex;

	Super.Tick(DeltaTime);

	if ( !bInitMapsLoaded )
	{
		bInitMapsLoaded=True;
		GotoState('Loading');
	}

	if ( Level.Game.NumPlayers > CurrentPlayerCount )
	{
		CurrentPlayerCount = Level.Game.NumPlayers;
	}

	if ( Level.Game.NumPlayers < CurrentPlayerCount )
	{
		CurrentPlayerCount = Level.Game.NumPlayers;
		DebugLog("CurrentPlayerCount decreased:" @ string(CurrentPlayerCount));
		CleanUpPlayerIDs();
		if ( Level.Game.NumPlayers > 0 )
		{
		    if (Level.Game.NumPlayers == 1 && TimeLeft > 1 && !bGameHasEnded)
		    {
		        TimeLeft = 0;
		        bMidGameTimeout = true;
		        TallyVotes(True);
		    }
		    else
		    {
			    TallyVotes(False);
			    UpdateOpenWRI();
			}
		}
	}

	if ( Level.Game.CurrentID > CurrentID )
	{
		if ( JoinTickCount < 20 )
		{
			JoinTickCount++;
			return;
		}
		JoinTickCount=0;

        // Find the new player
        for ( Other = Level.PawnList; Other != None; Other = Other.NextPawn )
        {
            if (Other.IsA('PlayerPawn') && (Other.PlayerReplicationInfo != None) && (Other.PlayerReplicationInfo.PlayerID == CurrentID) )
               break;
        }

		CurrentID++;

		if ( (Other == None) ||  !Other.bIsPlayer ||  !Other.IsA('PlayerPawn') )
		{
			return;
		}

		if ( Level.NetMode != NM_Standalone )
		{
		    sTemp = PlayerPawn(Other).GetPlayerNetworkAddress();
		    sTemp = Left(sTemp, InStr(sTemp, ":"));
		    Log("MapVoteX: New Player = "$Other.PlayerReplicationInfo.PlayerName$" | IP = "$sTemp$" | Time = "$GetAbsoluteTime());
		}
	}

	if (Level.Game.bGameEnded && !bGameHasEnded)
	    bGameHasEnded = true;
}

function ModifyPlayer( Pawn Other )
{
    Super.ModifyPlayer( Other );
    if (MVCI != None)
        MVCI.GameStart = 1;
    if (!bCheckRules)
        CheckRules();
}

function CheckRules()
{
    local int i;

    Level.Game.GetRules();
    if (Level.Game.EnabledMutators != "")
    {
        i = InStr(Level.Game.EnabledMutators, "MapVoteX103");
        if (i == -1)
            Level.Game.EnabledMutators = Level.Game.EnabledMutators$", MapVoteX103";
        bCheckRules = true;
    }
}

function HandleFailedMap(string MapName)
{
	local int i;

	MapName=GetPureMapname(MapName);
	Log("Map change Failed, bad or missing map file - MapName:" @ MapName);
    if (MVAM == None)
        MVAM = Spawn(class'MapVoteAdminMessages');
    MVAM.SaveAdminMessage("MapVoteX", "The map: "$Mapname$" has crashed due to failure or could not be found!");
    if (MVCM == None)
        MVCM = Spawn(class'MapVoteCrashedMaps');
    MVCM.StoreCrashedMap(MapName);
}

function bool CheckForTie ()
{
	local TeamInfo Best;
	local int i;
	local Pawn P;
	local Pawn BestP;
	local PlayerPawn Player;

	if ( Level.Game.IsA('Assault') )
	{
		return False;
	}
	if ( Level.Game.IsA('Domination') )
	{
		return False;
	}

	if ( Level.Game.IsA('TeamGamePlus') )
	{   // find best team
		for (i=0; i<TeamGamePlus(Level.Game).MaxTeams; i++)
		{
			if ( (Best == None) || (Best.Score < TeamGamePlus(Level.Game).Teams[i].Score) )
			{
				Best = TeamGamePlus(Level.Game).Teams[i];
			}
		}

		for (i=0; i<TeamGamePlus(Level.Game).MaxTeams; i++)
		{
			if ( (Best.TeamIndex != i) && (Best.Score == TeamGamePlus(Level.Game).Teams[i].Score) )
			{
				return True; // teams tied
			}
		}
	}
	else  // check for death match tie
	{
        // find individual winner
        for (P = Level.PawnList; P != None; P = P.nextPawn)
        {
            if (P.bIsPlayer && ((BestP == None) || (P.PlayerReplicationInfo.Score > BestP.PlayerReplicationInfo.Score)) )
               BestP = P;
        }
        // check for tie
        for (P = Level.PawnList; P != None; P = P.nextPawn )
        {
            if (P.bIsPlayer && (BestP != P) && (P.PlayerReplicationInfo.Score == BestP.PlayerReplicationInfo.Score) )
               return true; // tied
        }
	}
	return False;
}

function SubmitVote(string MapName, PlayerPawn Voter)
{
	local int PlayerIndex;
	local int X;
	local int MapIndex;

	if ( Voter.IsA('Spectator') &&  !CheckForAdmin(Voter) )
	{
		Voter.ClientMessage("Spectators are not allowed to vote.");
		return;
	}

	if ( bLevelSwitchPending )
	{
		return;
	}

	if ( Left(MapName,3) == "[X]" )
	{
		return;
	}

    MapName = GetPureMapname(MapName);
	Log(Voter.PlayerReplicationInfo.PlayerName $ " has voted for " $ MapName);

	CleanUpPlayerIDs();
	PlayerIndex = FindPlayerIndex(Voter.PlayerReplicationInfo.PlayerID,True);

	if ( PlayerIndex == -1 )
	{
		return;
	}

    MapIndex = GetMapIndex(MapName);

    if ( MapIndex == 0 )
    {
       return;
	}

	if ( PlayerVote[PlayerIndex] == MapIndex )
	{
	    return;
	}

	PlayerVote[PlayerIndex] = MapIndex;
    BroadcastMessage(Voter.PlayerReplicationInfo.PlayerName $ " voted for " $ MapName,True);
    TallyVotes(False);
}

function bool CheckForAdmin(PlayerPawn aPlayer)
{
    if ((aPlayer.PlayerReplicationInfo.bAdmin) || (aPlayer.bAdmin && Level.NetMode != NM_Standalone))
        return true;
    else
        return false;
}

function Mutate (string MutateString, PlayerPawn Sender)
{
	local string MapName, GamName;
	local string PlayerName;
	local int PlayerID;
	local int pos;
	local int seq;
	local int ObjectCount;
	local int i;
	local string CustSetting;

	Super.Mutate(MutateString,Sender);

    DebugLog("MutateString = "$MutateString$" | sCode = "$sCode);

    if (Left(string(Level.Game.Class), 7) ~= "ProDuel")
    {
       Super.Mutate(MutateString, Sender);
       return;
    }

	if ( Left(Caps(MutateString),11) == "PROVOTEMENU" || Left(Caps(MutateString),10) == "BDBMAPVOTE" )
	     MutateString = "MAPVOTEX VOTEMENU";

	if ( Left(Caps(MutateString),8) == "MAPVOTEX" )
	{
        if (!bMapsLoaded)
        {
            Sender.ClientMessage("Please wait while the maps are being loaded.");
            return;
        }
        MutateString = Mid(MutateString, 9);
		if ( Caps(MutateString) == "VOTEMENU" )
		{
			if ( (Level.TimeSeconds > GameStartVoteDelay) || (Level.NetMode == 0) || CheckForAdmin(Sender))
			{
				CleanUpPlayerIDs();
				OpenVoteWindow(Sender,True,True);
			}
			else
			{
				Sender.ClientMessage("Please wait "$GameStartVoteDelay$" seconds to vote");
			}
			return;
		}

        if (MutateString ~= "SpawnFiles" && CheckForAdmin(Sender))
        {
            SpawnAdvancedClasses(Sender);
            return;
        }

        if (Left(MutateString, 3) != sCode)
            return;

        MutateString = Mid(MutateString, 3);

		if ( Left(MutateString, 3) ~= "MAP" )
		{
			MapName = Mid(MutateString, 4);
			if (Left(MapName, 1) == "-")            // incase map prefix was ommited.
		        MapName = LoadedMapPrefix$MapName;
			if (CheckForAdmin(Sender))
			{
                MapName = GetPureMapname(MapName);
				BroadcastMessage("Server Admin has force a map switch to " $ MapName,True);
				ServerTravelString = SetupGameMap(MapName);
				CloseAllVoteWindows();
				Level.ServerTravel(ServerTravelString,False);
			}
			else
			{
				SubmitVote(MapName, Sender);
			}
			return;
		}

		if ( Left(MutateString, 4) ~= "GAME" )
		{
			GamName=Mid(MutateString, 5);
			if (CheckForAdmin(Sender))
			{
				BroadcastMessage("Server Admin has force a map switch to " $Mid(GamName, 3),True);
				ServerTravelString = SetupGameMap(GamName);
				CloseAllVoteWindows();
				Level.ServerTravel(ServerTravelString,False);
			}
			else
			{
				if (Mid(GamName, 3) == MVGameName)
				{
				   Sender.ClientMessage("You cannot vote for the current game! ");
				   return;
				}
				SubmitVote(GamName, Sender);
			}
			return;
		}

        if ( Left(MutateString, 12) ~= "AdminMessage" )
        {

             Sender.ClientMessage("Message for Admin Received." );
             if (MVAM == None)
                 MVAM = Spawn(class'MapVoteAdminMessages');
             MVAM.SaveAdminMessage(Sender.PlayerReplicationInfo.PlayerName, Mid(MutateString, 13));
             return;
        }

        if ( MutateString ~= "DisplayAdminMessages" )
        {
             bDisplayAdminMessages = true;
             if (MVAM == None)
                 MVAM = Spawn(class'MapVoteAdminMessages');
             OpenVoteWindow(Sender,True,True);
             return;
        }

        if ( MutateString ~= "DeleteAdminMessages" )
        {
             bDisplayAdminMessages = false;
             if (MVAM == None)
                 MVAM = Spawn(class'MapVoteAdminMessages');
             MVAM.DeleteMessages();
             Sender.ClientMessage("All Admin Messages have been deleted.");
             return;
        }

        if (CheckForAdmin(Sender))
        {
           if ( MutateString ~= "SetupServerMaintenance")
           {
              if (bMaintenanceMode)
              {
                  Sender.ClientMessage("Switching Off Maintenance Mode");
                  bMaintenanceMode = false;
                  Level.ConsoleCommand("set engine.gameinfo GamePassword "$ServerGamePassword);
                  SaveConfig();
                  if (MVTempName != "")
                      class'Engine.GameReplicationInfo'.default.ServerName = MVTempName;
                  class'Engine.GameReplicationInfo'.static.StaticSaveConfig();
                  Log("Restarting Server with Maintenance Mode Off", 'MapVoteX');
                  Level.ServerTravel("?restart", false);
                  return;
              }
              else
              {
                  Sender.ClientMessage("Switching On Maintenance Mode");
                  bMaintenanceMode = true;
                  MVTempName = class'Engine.GameReplicationInfo'.default.ServerName;
                  SaveConfig();
                  if (MaintenancePassword != "")
                      Level.ConsoleCommand("set engine.gameinfo GamePassword "$MaintenancePassword);
                  if (MaintenanceServerName != "")
                      class'Engine.GameReplicationInfo'.default.ServerName = MaintenanceServerName;
                  class'Engine.GameReplicationInfo'.static.StaticSaveConfig();
                  class'Engine.GameInfo'.static.StaticSaveConfig();
                  Log("Restarting Server with Maintenance Mode On", 'MapVoteX');
                  Level.ServerTravel("?restart", false);
                  return;
              }
           }
           else
           if ( MutateString ~= "ForceGameEnd" )
           {
              BroadcastMessage( Sender.PlayerReplicationInfo.PlayerName @ "forced the game to end." );
              Level.Game.EndGame( "forced by admin" );
           }
           else
           if ( MutateString ~= "PauseGame" )
           {
              if (Level.Pauser == "")
              {
 		         Level.Pauser = Sender.PlayerReplicationInfo.PlayerName;
		         BroadCastMessage("Game paused by Admin.");
              }
              else
              {
		         Level.Pauser = "";
		         BroadCastMessage("Game restarted by Admin.");
              }
           }
           else
           if ( MutateString ~= "AddBot" )
           {
              if (DeathMatchPlus(Level.Game).NumBots < 16)
              {
                 Level.Game.ForceAddBot();
                 BroadcastMessage("Admin added a Bot to the game.");
                 return;
              }
              Sender.ClientMessage("Bots in game are already at maximum.");
              return;
           }
           else
           if ( MutateString ~= "RemBot" )
           {
              if (RemoveBot())
                 BroadcastMessage("Admin removed a Bot from the game.");
           }
           else
           if ( Left(MutateString, 5) ~= "Admin" )
           {
              SetAdminStuff(Mid(MutateString, 6));
              Sender.ClientMessage("Game Configurations have been updated.");
           }
        }
	}
}
//autoopen,midgamevoting,debugmode,!cmds,bmajority,logonsplash,advertising,bwelcomefirst,sortgamelist,emptyserverswitch,
//emptyservertime,startvotedelay,votetimelimit,menupopup,repeatlimit
function SetAdminStuff(string Stuff)
{
    local int i;
    local string sTemp;

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bAutoOpen = bool(i);
    default.bAutoOpen = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bAllowMidGameVoting = bool(i);
    default.bAllowMidGameVoting = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bDebugMode = bool(i);
    default.bDebugMode = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bUseCommandOptions = bool(i);
    default.bUseCommandOptions = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bMajorityPlayerVote = bool(i);
    default.bMajorityPlayerVote = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bEnableSplashLogo = bool(i);
    default.bEnableSplashLogo = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bEnableWelcomeMenu = bool(i);
    default.bEnableWelcomeMenu = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bShowWelcomeFirst = bool(i);
    default.bShowWelcomeFirst = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bSortCustomGameList = bool(i);
    default.bSortCustomGameList = bool(i);

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    bEnableEmptyServerSwitch = bool(i);
    default.bEnableEmptyServerSwitch = bool(i);

    //---------------- sliders -------------------

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    EmptyServerSwitchTimeMins = i;
    default.EmptyServerSwitchTimeMins = i;

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    GameStartVoteDelay = i;
    default.GameStartVoteDelay = i;

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    VoteTimeLimit = i;
    default.VoteTimeLimit = i;

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    Stuff = Mid(Stuff, i+1);
    i = int(sTemp);
    ScoreBoardDelay = i;
    default.ScoreBoardDelay = i;

    i = InStr(Stuff, ",");
    sTemp = Left(Stuff, i);
    i = int(sTemp);
    RepeatLimit = i;
    default.RepeatLimit = i;

    SaveConfig();
}

function string GetAdminStuff()
{
    local string sTemp, s0, s1;

    s0 = "0,";
    s1 = "1,";

    if (bAutoOpen)               // wouldn't work any other way ?
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bAllowMidGameVoting)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bDebugMode)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bUseCommandOptions)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bMajorityPlayerVote)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bEnableSplashLogo)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bEnableWelcomeMenu)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bShowWelcomeFirst)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bSortCustomGameList)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    if (bEnableEmptyServerSwitch)
        sTemp = sTemp$s1;
    else
        sTemp = sTemp$s0;

    //---------------- sliders -------------------

    sTemp = sTemp$string(EmptyServerSwitchTimeMins)$","$string(GameStartVoteDelay)$","$string(VoteTimeLimit)$","$
            string(ScoreBoardDelay)$","$string(RepeatLimit)$",";

    return sTemp;
}

function OpenVoteWindow (PlayerPawn Sender, bool bSpecsAllowed, bool bManualOpen)
{
	local MapVoteWRI MVWRI, A;
	local MapVoteMessageWRI MMWRI;
	local int i, j, X, Y;
	local int PlayerCount;
	local Pawn P;
	local int TeamID;
	local string sTemp;
	local bool bDone;

	if ( Sender.IsA('Spectator') &&  !CheckForAdmin(Sender) &&  !bSpecsAllowed )
	{
		return;
	}

	if ( bLevelSwitchPending )
	{
		return;
	}

    if (bDisplayAdminMessages)
    {
        MMWRI = Spawn(Class'MapVoteMessageWRI',Sender,,Sender.Location);

        if (MVAM == None)
            MVAM = Spawn(class'MapVoteAdminMessages');

        MMWRI.sCode = sCode;

        X=1;
	    for (i=0; i<100; i++)
	    {
		    if (MVAM.Message[i] == "")
		        break;
	        MMWRI.Mesglist[X] = MVAM.Message[i];
	        X++;
	    }
        bDisplayAdminMessages = false;
        return;
    }

	foreach AllActors(Class'MapVoteWRI',A)
	{
		if ( Sender == A.Owner )
		{
			return;
		}
	}

    GetServerInfo();             // ip and stuff
    sTemp = GetAdminStuff();

	MVWRI = Spawn(Class'MapVoteWRI',Sender,,Sender.Location);
	if ( MVWRI == None )
	{
		Log("#### -- PostLogin :: Fail:: Could not spawn WRI");
		return;
	}

    bDone = false;

    MVWRI.MyBool[1] = int(CheckForAdmin(Sender));
    MVWRI.MyBool[2] = int(bGameHasEnded);
    MVWRI.MyBool[3] = int(bEnableWelcomeMenu);
    MVWRI.MyBool[4] = int(bShowWelcomeFirst);
    MVWRI.MyBool[5] = int(bMaintenanceMode);
    MVWRI.MyBool[6] = int(bAllowMidGameVoting);

    MVWRI.MyData[1] = sCode;
    MVWRI.MyData[2] = Version;

    if (CheckForAdmin(Sender))
    {
        MVWRI.MyData[3] = sTemp;
    }
    sTemp = ServerName;
    if (Len(sTemp) > 50)
        sTemp = Left(sTemp, 50);                      // ..:: PSYCHO | *NewNet* |
    MVWRI.MyData[4] = sTemp;
    MVWRI.MyData[5] = ServerPort;
    MVWRI.MyData[6] = ServerIP;

    if (bEnableWelcomeMenu)
    {
        if (MVCW == None)
            MVCW = Spawn(class'MapVoteClientWelcome');
        MVWRI.MyData[7]  = MVCW.GetItem(LoadedGameNum, 1); // stretch
        MVWRI.MyData[8]  = MVCW.GetItem(LoadedGameNum, 2); // logoname
        MVWRI.MyData[9]  = MVCW.GetItem(LoadedGameNum, 3); // soundname
        MVWRI.MyData[10] = MVCW.GetItem(LoadedGameNum, 4); // website url
        MVWRI.MyData[11] = MVCW.GetItem(LoadedGameNum, 5); // contact url
        MVWRI.MyData[12] = MVCW.GetItem(LoadedGameNum, 6); // anon button name
        MVWRI.MyData[13] = MVCW.GetItem(LoadedGameNum, 7); // anon button url
    }

    MVWRI.MyData[14] = LoadedMapPrefix;

	PlayerCount=0;
	for (P = Level.PawnList; P != None; P = P.NextPawn)
	{
		if ((PlayerPawn(P) != None) && P.bIsPlayer)
		{
			if ( !P.IsA('Spectator') )
			{
 			    TeamID = 0;
 			    if (Level.Game.bTeamGame)
 			        TeamID = PlayerPawn(P).PlayerReplicationInfo.Team + 1;
			    if ( PlayerVote[FindPlayerIndex(PlayerPawn(P).PlayerReplicationInfo.PlayerID, True)] > 0 )
			    {
				    MVWRI.PlayerName[PlayerCount++ ] = string(TeamID)$PlayerPawn(P).PlayerReplicationInfo.PlayerName;
				}
			}
		}
	}

    for (X=0; X<31; X++)
    {
 	    if (MapStatusText[X] != "")
	    {
		    MVWRI.MapVoteResults[X] = MapStatusText[X];
	    }
	}

	MVWRI.MapVoteResults[X]= "";
}

function bool RemoveBot()
{
    local Bot DumbBot;
    local Pawn P, OtherPawn;
    local actor A;

    for (P = Level.PawnList; P != None; P = P.NextPawn)
    {
        if (Bot(P) != None) // get Bot with least score
           if (DumbBot == None || P.PlayerReplicationInfo.Score < DumbBot.PlayerReplicationInfo.Score)
               DumbBot = Bot(P);
    }

    if (DumbBot != None)
    {
        DumbBot.Destroy();
        DeathMatchPlus(Level.Game).MinPlayers -= 1;
        return true;
    }

    return false;
}

function int FindPlayerIndex (int PlayerID, bool bAddIfNotFound)
{
	local int i;

    for (i=0; i<32; i++)
	{
		if ( PlayerIDList[i] == PlayerID )
		{
			return i;
		}
	}

	if ( bAddIfNotFound )
	{
       for (i=0; i<32; i++)
	   {
		    if ( PlayerIDList[i] == -1 )
			{
				PlayerIDList[i] = PlayerID;
				return i;
			}
		}
	}
	return -1;
}

function CleanUpPlayerIDs()
{
	local Pawn aPawn;
	local int x;
	local bool bFound;

    for (x=0; x<32; x++)
    {
        if (PlayerIDList[x] >- 1)
        {
           bFound = false;
           for ( aPawn=Level.PawnList; aPawn!=None; aPawn=aPawn.NextPawn )
           {
               if (aPawn.bIsPlayer && aPawn.IsA('PlayerPawn') && PlayerPawn(aPawn).PlayerReplicationInfo.PlayerID == PlayerIDList[x])
               {
                   bFound = true;
                   break;
               }
           }
			 if ( !bFound )
			 {
				 DebugLog("Removing Player that has left:" @ string(PlayerIDList[x]));
				 PlayerVote[x]=0;
				 PlayerIDList[x]=-1;
	       }
        }
    }
}

function ClearAllVoting()
{
    local int i;

    for (i=0; i<32; i++)
    {
         PlayerVote[i] = 0;
         MapStatusText[i] = "";
    }
}

function TallyVotes (bool bForceMapSwitch)
{
    local Pawn P;
	local Actor A;
	local int Index;
	local int i, x, y;
	local int topmap;
	local int textline;
	local int TieCount;
	local string sTemp;
	local string MapName;
	local string GameType;
	local string CurrentMap;
	local float VoteCount[550];   // 500 maps - 30 games
	local int Ranking[32];
	local int PlayersThatVoted;
	local int PlayersLeftToVote;

	if ( bLevelSwitchPending )
	{
		return;
	}

    DebugLog("0  Tally Top bForceMapSwitch = "$bForceMapSwitch);

	PlayersThatVoted=0;
	for (x=0; x<32; x++)
	{
		if ( PlayerVote[x] != 0 )
		{
			PlayersThatVoted++;
		    VoteCount[PlayerVote[x]] += GetMapPriority(MapList[PlayerVote[x]]);
			DebugLog("PlayerVote["$x$"]:" @ string(PlayerVote[x]));
			DebugLog("VoteCount[PlayerVote["$x$"]]:" @ string(VoteCount[PlayerVote[x]]));
		}
	}

	if (!Level.Game.bGameEnded && !bMidGameVote && PlayersThatVoted > 0)// && (float(PlayersThatVoted) / float(Level.Game.NumPlayers)) * 100 >= 51)
	{
		BroadcastMessage("Mid-Game Map Voting has been initiated !!!!");
		bMidGameVote = True;
		TimeLeft = VoteTimeLimit;
		//ScoreBoardTime = 1;
		SetTimer(1.00, True);
		DebugLog("1  Tally Vote % = "$float(PlayersThatVoted) / float(Level.Game.NumPlayers) * 100$" | PlayersThatVoted = "$PlayersThatVoted$" | Level.Game.NumPlayers = "$Level.Game.NumPlayers$" | TimeLeft = "$TimeLeft);
	}

	Index=0;
	for (x=1; x<=MapCount; x++)     // includes GameCount
	{
		if ( VoteCount[x] > 0 )
		{
			Ranking[Index++ ] = x;  // copy all map indexes to the ranking list if someone has voted it.
		}
	}

    // bubble sort ranking list by vote count
    for (x=0; x<index-1; x++)
    {
        for (y=x+1; y<index; y++)
        {
            if (VoteCount[Ranking[x]] < VoteCount[Ranking[y]])
            {
               topmap = Ranking[x];
               Ranking[x] = Ranking[y];
               Ranking[y] = topmap;
            }
        }
    }

	PlayersLeftToVote = Level.Game.NumPlayers - PlayersThatVoted;
	DebugLog("2  Tally NumPlayer:" $ string(Level.Game.NumPlayers) @ "PlayerVoted:" $ string(PlayersThatVoted) @ "PlayersToVote:" $ string(PlayersLeftToVote));

    //if (bMajorityPlayerVote && Level.Game.bGameEnded && (PlayersThatVoted > (Level.Game.NumPlayers / 2)))
    if (bMajorityPlayerVote && (PlayersThatVoted > (Level.Game.NumPlayers / 2)))
    {
        PlayersLeftToVote = 0;
        DebugLog("3  Tally Majority votes forcing PlayersLeftToVote = 0");
    }

    if ((PlayersLeftToVote < VoteCount[Ranking[0]]) && (VoteCount[Ranking[1]] + PlayersLeftToVote * GetMapPriority(MapList[Ranking[1]]) < VoteCount[Ranking[0]]) && (VoteCount[Ranking[2]] + PlayersLeftToVote * GetMapPriority(MapList[Ranking[2]]) < VoteCount[Ranking[0]]) && (VoteCount[Ranking[3]] + PlayersLeftToVote * GetMapPriority(MapList[Ranking[3]]) < VoteCount[Ranking[0]]) && Level.Game.bGameEnded )
	{
		bForceMapSwitch = True;
		DebugLog("4  Tally bForceMapSwitch = "$bForceMapSwitch);
	}

    //Update Status Page
    for (x=0; x<index; x++)
	{
		MapStatusText[X] = GetPureMapname(MapList[Ranking[x]]) $ "," $ Left(string(VoteCount[Ranking[x]]),3);
		//MapStatusText[x] = MapList[Ranking[x]] $ "," $ VoteCount[Ranking[x]]; //bdb3
	}
	MapStatusText[Index]= "";

	UpdateOpenWRI();

	//Check for a tie
	if ( (VoteCount[Ranking[0]] == VoteCount[Ranking[1]]) && (VoteCount[Ranking[0]] != 0) )
	{
	    TieCount = 1;
        for (x=1; x<index; x++)
        {
            if (VoteCount[Ranking[0]] == VoteCount[Ranking[x]])
               TieCount++;
        }

	    topmap = Ranking[Rand(TieCount)];
	    // Don't allow same map to be choosen
		CurrentMap = RemovePrefix(GetURLMap());
        DebugLog("5  Tally Topmap = "$Topmap$" | CurrentMap = "$CurrentMap);
		if ( InStr(CurrentMap,".") > -1 )
		{
			CurrentMap = Left(CurrentMap,InStr(CurrentMap,"."));
		}

        x = 0;
        while (MapList[topmap] ~= CurrentMap)
        {
              topmap = Ranking[Rand(TieCount)];
              x++;
              if (x > 20)
                 break;
        }
	}
	else
	{
		topmap = Ranking[0];
	}

	if ( bForceMapSwitch )
	{
		if ( (PlayersThatVoted == 0) || (Level.Game.NumPlayers == 0) )
		{
			   BroadcastMessage("No votes received, choosing a random map...",True);
			   topmap = GetRandomMap();
		}
	}

	if ( bForceMapSwitch || (Level.Game.NumPlayers == PlayersThatVoted) )
	{
		if ( MapList[topmap] == "" )
		{
		    DebugLog("7  Tally Topmap = NULL - Return");
			return;
		}

        if ( bMidGameTimeout )
        {
			// clear everything and close windows.
			bMidGameTimeout = false;
			bForceMapSwitch = false;
			bMidGameVote = false;
			ClearAllVoting();
			CloseAllVoteWindows();
		    if ( Level.Game.NumPlayers == PlayersThatVoted )
 		        BroadcastMessage("Tie:  Midgame votes have been cancelled.", True);
		    else
		        BroadcastMessage("Timeout: Midgame votes have been cancelled.", True);
		    DebugLog("9  Tally - Tie:  Midgame votes have been cancelled.");
		    return;
        }

		sTemp = MapList[topmap];
		if (Left(sTemp, 3) == "[G]")
		    sTemp = Mid(sTemp, 3);

		if ( (VoteCount[Ranking[0]] == VoteCount[Ranking[1]]) && (VoteCount[Ranking[0]] != 0) )
		{
			if (!Level.Game.bGameEnded)
			{
		       DebugLog("8  Tally - TimeLeft = "$TimeLeft$" | No vote has won - Exit");
		       return;
			}
			BroadcastMessage("Tie: "$GetPureMapname(sTemp) $ " has won.", True);
			DebugLog("10 Tally Vote drew a tie, but the Map: "$sTemp$" has won.");
		}
		else
		{
			BroadcastMessage(GetPureMapname(sTemp) $ " has won.", True);
		}

		CloseAllVoteWindows();
		bLevelSwitchPending = True;
		ServerTravelString = SetupGameMap(MapList[topmap]);
		ServerTravelTime = Level.TimeSeconds + 4;
		if (RepeatLimit > 0)
            MVRP.AddMap(LoadedGameNum, MapList[topmap]);

		if ( !Level.Game.bGameEnded )
		{
			SetTimer(1.00,True);
		}

		if ( (PlayersThatVoted == 0) || (Level.Game.NumPlayers == 0) )
		{
			return;
		}
	}
	DebugLog("11 Tally - Exit");
}

function UpdateOpenWRI()
{
	local MapVoteWRI MVWRI;
	local Pawn P;
	local int X, Z;
	local int TeamID;

	foreach AllActors(Class'MapVoteWRI',MVWRI)
	{
		Z = 0;
		for (P = Level.PawnList; P != None; P = P.NextPawn)
		{
			if ((PlayerPawn(P) != None) && P.bIsPlayer)
			{
				if ( !P.IsA('Spectator') )
				{
 	 			    TeamID = 0;
 	 			    if (Level.Game.bTeamGame)
 	 			        TeamID = PlayerPawn(P).PlayerReplicationInfo.Team + 1;
				    if ( PlayerVote[FindPlayerIndex(PlayerPawn(P).PlayerReplicationInfo.PlayerID, True)] > 0 )
				    {
					    MVWRI.PlayerName[Z++] = string(TeamID)$PlayerPawn(P).PlayerReplicationInfo.PlayerName;
					}
				}
			}
		}

		MVWRI.UpdateMapVoteResults("Clear", X);
		for (X=0; X<31; X++)
		{
		    if (MapStatusText[X] != "")
 		        MVWRI.UpdateMapVoteResults(MapStatusText[X], X);
		}
		MVWRI.UpdateMapVoteResults("", X);
	}
}

function float GetMapPriority(string MapName)
{
	local int MapListNum;

	if ( MapName == "" )
		return 0.00;
	else
	    return 1.00;
}

function string GetPureMapname (string MapName)
{
	if ( Left(MapName,3) ~= "[x]" )
	{
		MapName=Mid(MapName,3);
	}
	if ( Left(MapName,3) ~= "[G]" )
	{
		MapName=Mid(MapName,3);
	}
	if ( Left(Right(MapName,2),1) == ":" )
	{
		MapName=Left(MapName,Len(MapName) - 2);
	}
	return MapName;
}

function CloseAllVoteWindows ()
{
	local MapVoteWRI MVWRI;

	foreach AllActors(Class'MapVoteWRI',MVWRI)
	{
		MVWRI.CloseWindow();
		MVWRI.Destroy();
	}
}

function HandleDefaultSwitch()
{
    local string sTemp;
    local int i;

    i = DefaultCustomGameNum;
    if (i < 0 || i > 29)
        i = 0;

    if (MVGameName != CustomGame[i].GameName)
    {
        if (CustomGame[i].bUse == False || CustomGame[i].DefaultMap == "" || CustomGame[i].GameClass == "" || CustomGame[i].GameName == "")
        {
            Log("Empty Server Switch cannot switch due to error in Default CustomGame["$i$"]", 'MapVoteX');
            return;
        }
        ServerTravelString = SetupGameMap("[G]"$CustomGame[i].GameName);
    }
    else
    if (LoadedMapName == CustomGame[i].DefaultMap && !bMaintenanceMode)
        return;
    else
    if (bEmptyServerSwitchMap || bMaintenanceMode)
        ServerTravelString = SetupGameMap(CustomGame[i].DefaultMap);
    else
        return;

    if (bMaintenanceMode)
    {
       bMaintenanceMode = false;
       SaveConfig();
       Level.ConsoleCommand("set engine.gameinfo GamePassword "$ServerGamePassword);
       if (MVTempName != "")
           class'Engine.GameReplicationInfo'.default.ServerName = MVTempName;
       class'Engine.GameReplicationInfo'.static.StaticSaveConfig();
    }

    Level.ServerTravel(ServerTravelString, False);
    Log("Empty Server Switch at: "$GetAbsoluteTime()$" to: "$CustomGame[i].GameName, 'MapVoteX');
}

function string SetupGameMap (string MapName)
{
    local string sTemp;
	local string GameType;
	local string GameName;
	local bool bGame;
	local int i, j, iNum;

    if (Left(Mapname, 3) == "[G]")
       bGame = true;

    MapName=GetPureMapname(MapName);
	ResetAssault();

    iNum = -1;

    GameType = string(Level.Game.Class);
    GameName = Level.Game.GameName;

	if ( MapName == "" )
	{
		return "?Restart";
	}

    if (bGame)
	{
	    sTemp = MapName;
	    MapName = "";
	    for (i=0; i<CustomList; i++)
	    {
	        if (CustomGame[i].GameName ~= sTemp)
	        {
	           iNum = i;
	           GameType = CustomGame[i].GameClass;
	           MapName  = CustomGame[i].DefaultMap;
               j = InStr(Caps(MapName),".UNR");
               if ( j != -1 )
		          MapName = Left(MapName, j);
	           GameName  = sTemp;
	           LoadedMapPrefix = GetMapPrefix(MapName);
	           break;
	        }
	    }
        if (iNum >= 0)
        {
            if (!bUseCustomGameMapLists)
                SetMapPath(CustomGame[iNum].MapFolder);
            SaveServerName(iNum);
            SavePackageActors(iNum);
        }
	}
	else
	{
	     for (i=0; i<CustomList; i++)
	     {
	          if (CustomGame[i].GameName == MVGameName)
	          {
	              iNum = i;
	              GameType = CustomGame[i].GameClass;
	              GameName = CustomGame[i].GameName;
	              break;
	          }
	     }
	}

    SaveGameInfo(GameName);
	SaveConfig();
	TravelMap = MapName;
	return MapName $ ".unr?Game=" $ GameType;
}

state Loading
{
Begin:

	MapCount = 0;

    if ( !bLevelSwitchPending && !bRestarting )
    {
		for ( m=0; m<550; m++ )
		{
			MapList[m] = "";
		}

		for (m=0; m<CustomList; m++)
		{
			if ( CustomGame[m].bUse && (CustomGame[m].GameClass != "") && (CustomGame[m].GameName == MVGameName))
			{
				Log("Loading maps for: "$CustomGame[m].GameName, 'MapVoteX');
				CustomGameClass=Class<GameInfo>(DynamicLoadObject(CustomGame[m].GameClass,Class'Class'));
				if ( Right(CustomGame[m].GameClass,12) ~= "MultiCTFGame" )
				{
					DebugLog("MultiCTF detected. Fixing Mapprefix to CTFM");
					CustomGameClass.Default.MapPrefix="CTFM";
				}
				if ( CustomGameClass == None )
				{
					Log("MapVote: Error. CustomGame[" $ string(m) $ "]: Failed to load GameClass:" @ CustomGame[m].GameClass);
					break;
				}
				else
				{
					if (CustomGame[m].DefaultMap != "")
					{
						NewPrefix = FixPrefix(GetMapPrefix(CustomGame[m].DefaultMap));
					}
					else
					{
						NewPrefix = FixPrefix(CustomGameClass.default.MapPrefix);
					}

					Level.Game.GameName = CustomGame[m].GameName;

					if (MVML == None && !bLevelSwitchPending)
					{
					    bMapsLoaded = false;
					    MVML = Spawn(class'MapVoteMapLoader');
                        MVML.MyMut = self;
			            MVML.Prefix = NewPrefix;
                        if (bUseCustomGameMapLists)
                        {
                            MVML.StartLoadingMaps(NewPrefix, true, m);
					    }
					    else
					    {
				            MVML.StartLoadingMaps(NewPrefix, false, 0);
				        }
					}
					Sleep(0.00);
					break;
				}
				Sleep(0.00);
			}
		}
    }
}

function SortMapsAndGames() // called from maploading class
{
    SortMapList();
    LoadGameList();
    LoadGameReplication();
    bMapsLoaded = true;
    Log("MapVoteX: Total Maps Loaded = " $ string(MapCount-GameCount));
}

// Alphatise the maplist
function SortMapList()
{
   local int a, b;
   local string TempMapName;

   // bubble sort the map list
   for(a=1; a<=MapCount-1; a++)    // games load after this
   {
      for(b=a+1; b<=MapCount; b++)
      {
         if(Caps(MapList[a]) > Caps(MapList[b]))
         {
            TempMapName = MapList[a];
            MapList[a] = MapList[b];
            MapList[b] = TempMapName;
         }
      }
   }
}

function LoadGameList()
{
    local string sGame, sMap;
    local int i, j, k;

    for (i=0; i<CustomList; i++)
    {
        GameList[i] = "";
        GameMaps[i] = "";
    }

    k = 0;
	for (i=0; i<CustomList; i++)
	{
	    if (CustomGame[i].bUse && CustomGame[i].GameName != "")
	    {
		    GameList[k] = "[G]"$CustomGame[i].GameName;
		    sMap = CustomGame[i].DefaultMap;
            j = InStr(Caps(sMap),".UNR");
            if ( j != -1 )
                sMap = Left(sMap, j);
		    if (sMap == "")
		        sMap = "None";
		    GameMaps[k] = sMap;
		    GameCount++;
		    k++;
		}
	}

   if (bSortCustomGameList)
   {
      // bubble sort the game list
      for(i=0; i<=GameCount-1; i++)
      {
         for(j=i+1; j<=GameCount; j++)
         {
            if(Caps(GameList[i]) > Caps(GameList[j]))
            {
               sGame       = GameList[i];
               GameList[i] = GameList[j];
               GameList[j] = sGame;

               sMap        = GameMaps[i];
               GameMaps[i] = GameMaps[j];
               GameMaps[j] = sMap;
            }
         }
      }
   }

   if (GameList[0] == "" && GameList[1] != "")  // game sorting moves all up 1
   {
       for (i=0; i<30; i++)
       {
            GameList[i] = GameList[i+1];
            GameMaps[i] = GameMaps[i+1];
       }
   }

   j = 0;
   for (i=0; i<550; i++)
   {
       if (i > 0 && MapList[i] == "" && GameList[j] != "" && j < CustomList)
       {
           MapList[i] = GameList[j]; // use at tallyvote only
           MapCount++;
           j++;
       }
   }

}

function LoadGameReplication()
{
    local string sTemp;
    local int i, j, X;

	if (MVGRI == None)
        MVGRI = Spawn(class'MapVoteGRI');

    MVGRI.sCode   = sCode;

    j = 1;
    // do the game maps 1st
    for (i=0; i<30; i++)
    {
        MVGRI.GameStuff[j] = GameList[i];
        MVGRI.GameStuff[j+30] = GameMaps[i];
        j++;
    }

    MVGRI.GameStuff[62] = MVGameName;
    MVGRI.GameStuff[63] = LoadedMapPrefix;
    MVGRI.GameStuff[64] = string(GameCount);

    // do the map names
    j = Len(LoadedMapPrefix);
	for (i=0; i<500; i++)
	{
		if (MapList[i] != "")
		{
		   sTemp = MapList[i];
		   if (Left(sTemp, 3) != "[G]")
		   {
		      sTemp = Mid(sTemp, j+1); // skip the '-'
		      //if (MapList[i] == LoadedMapName || (RepeatLimit > 0 && MVRP.CheckMap(LoadedGameNum, MapList[i])))
		      if (MapList[i] == LoadedMapName)
		      {
		         if (RepeatLimit > 0)
		         {
		             if (MVRP.CheckMap(LoadedGameNum, MapList[i]))
		                 sTemp = "[X]"$sTemp;
		         }
		      }
		      if (X < 256)
		      {
		          MVGRI.ListedMaps1[X] = sTemp;
		      }
		      else
		      {
		          MVGRI.ListedMaps2[X-255] = sTemp;
		      }
		      X++;
		   }
		}
		if ((i > (MapCount + 2)) && MapList[i] == "")
		   break;
	}
}

function string FixPrefix (string Prefix)
{
	if (Right(Prefix, 1) == "-")
		return Left(Prefix,Len(Prefix) - 1);
	else
		return Prefix;
}

function string RemovePrefix (string MapName)
{
	if (Left(MapName,7) ~= "CTF-BT-")
	    return Mid(MapName, 7);
	return Mid(MapName,InStr(MapName, "-") + 1);
}

function string GetMapPrefix(string MapName)
{
    local int j;

	if (Left(MapName,7) ~= "CTF-BT-")
		return "CTF-BT";

    j = InStr(MapName, "-");
    if (j == -1)
        return "";
    else
        return Left(MapName, j);
}

function SetMapPath(string MapPath, optional bool bAll)
{
    local string sTemp, sPath, sFinal;
    local string Paths[50];
    local int i, j, k, x;

    for (i=0; i<50; i++)
         Paths[i] = "";

    if (Caps(MapPath) == "MAPS" || Caps(MapPath) == "MAPS/" || Caps(MapPath) == "../MAPS")
        MapPath = "";

    x = 5;
    if (bAll)
    {
        MapPath = GetAllPaths();
        if (MapPath == "")
            return;
        x = 50;
    }

    if (MapPath != "")
    {
        if (Right(MapPath, 1) != ",")
            MapPath = MapPath$",";
        for (i=0; i<x; i++)
        {
             j = InStr(MapPath, ",");
             if (j == -1)
                 break;
             Paths[i] = Left(MapPath, j);
             MapPath = Mid(MapPath, j+1);
        }
    }

    sPath = Level.ConsoleCommand("Get Core.System Paths");
    i = InStr(Caps(sPath), "../SYSTEM/*.U");
    if (i > 5)
    {
	    Log("System.Core Failure! Paths=../System/*.u is not first. Cannot write to file!", 'MapVoteX');
	    Log("Current Order: "$sPath, 'MapVoteX');
        if (MVAM == None)
            MVAM = Spawn(class'MapVoteAdminMessages');
        MVAM.SaveAdminMessage("MapVoteX", "System.Core Failure! Paths=../System/*.u is not first. Cannot write to file!");
    }

    j = InStr(sPath, ")");
    if (j >= 0)
       sPath = Left(sPath, j);
    sPath = sPath$",";
    sFinal = "";
    for (i=0; i<50; i++)
    {
        j = InStr(sPath, ",");
        if (j < 0)
            break;
        sTemp = Left(sPath, j);
        sPath = Mid(sPath, j+1);
        k = InStr(Caps(sTemp), ".UNR");
        if (k < 0)
        {
           sFinal = sFinal$sTemp$",";
        }
        else
        {
           k = InStr(Caps(sTemp), "MAPS/*.UNR");
           if (k > 0)
              sFinal = sFinal$sTemp$",";
        }
    }

    for (i=0; i<x; i++)
    {
        if (Paths[i] != "")
        {
            sTemp = "\"../"$Paths[i]$"/*.unr\",";
            j = InStr(Caps(sFinal), Caps(sTemp));
            if (j == -1)
                sFinal = sFinal$sTemp;
        }
    }

    if (Right(sFinal, 1) == ",")
        sFinal = Left(sFinal, Len(sFinal)-1);
    sFinal = sFinal$")";

    Level.ConsoleCommand("set Core.System Paths "$sFinal);
    Level.Game.SaveConfig();
    DebugLog("New Map Paths = "$sFinal);
}

function string GetAllPaths()
{
     local string sTemp, sPath;
     local int i, j;

     sPath = "";

     for (i=0; i<30; i++)
     {
         if (CustomGame[i].bUse == True)
         {
            sTemp = CustomGame[i].MapFolder;
            if (sTemp != "")
            {
                if (Caps(sTemp) != "MAPS" && Caps(sTemp) != "MAPS/")
                {
                    j = InStr(Caps(sPath), Caps(sTemp));
                    if (j == -1)
                        sPath = sPath$sTemp$",";
                }
            }
         }
     }
     return sPath;
}
function DebugLog (string Msg)
{
	if ( bDebugMode )
	{
		Log("**** " $Msg);
	}
}

function int ResetAssault()
{
	local int CurrentDefender;

	if ( Level.Game.IsA('Assault') )
	{
		if ( GetWinningTeam() != -1 )
		{
			Log("MapVoteX: Team " $ string(GetWinningTeam()) $ " won the round.");
			if ( GetWinningTeam() == 1 )
			{
				CurrentDefender=0;
			}
			else
			{
				CurrentDefender=1;
			}
			Log("MapVoteX: Setting Team " $ string(CurrentDefender) $ " as the defending team for the next map.");
		}
		else
		{
			CurrentDefender=0;
		}
		Assault(Level.Game).ResetGame();
		Assault(Level.Game).bDefenseSet=False;
		Assault(Level.Game).NumDefenses=0;
		Assault(Level.Game).SavedTime=0.00;
		Assault(Level.Game).GameCode="";
		Assault(Level.Game).Part=1;
		Assault(Level.Game).bTiePartOne=False;
		Assault(Level.Game).CurrentDefender=CurrentDefender;
		Assault(Level.Game).SaveConfig();
	}
	return 0;
}

function int GetWinningTeam ()
{
	if ( Assault(Level.Game).bDefenseSet )
	{
		if ( Assault(Level.Game).bAssaultWon )
		{
			return Assault(Level.Game).Attacker.TeamIndex;
		}
		else
		{
			if (  !Assault(Level.Game).bTiePartOne )
			{
				return Assault(Level.Game).Defender.TeamIndex;
			}
			else
			{
				return -1;
			}
		}
	}
	else
	{
		return -1;
	}
}

function int GetMapIndex (string MapName)
{
	local int i;
	local int MapIndex;

    MapName = GetPureMapname(MapName);

	i=1;
	for (i=1; i<=MapCount; i++)      // includes GameCount
	{
	   if ( GetPureMapname(MapList[i]) ~= MapName )
	   {
		  MapIndex=i;
	   }
	}

	return MapIndex;
}

function int GetRandomMap()
{
	local int i, R;

    for (i=0; i<100; i++)
    {
        R = Rand(MapCount - GameCount);
	    if (R > 1)
	       break;
	}
	if (R < 1)
	    R = 1;
	return R;
}

function bool StartMutatorsAndSettings()
{
	local string Mutators;
	local string Settings;
	local string sTemp, sMap;
	local int i, j, pos;
	local bool bFound;

	if ( Right(string(Level.Game.Class),12) ~= "MultiCTFGame" )
	{
		DebugLog("MultiCTF detected. Fixing Mapprefix to CTFM");
		Level.Game.Default.MapPrefix="CTFM";
		Level.Game.MapPrefix="CTFM";
	}

    j = InStr(Caps(LoadedMapName), "-TUTORIAL");  // server startup ?
    if (j >= 0)
    {
       i = DefaultCustomGameNum;
       if (i < 0 || i > 29)
           i = 0;
       bMaintenanceMode = false;
       default.bMaintenanceMode = false;
       SaveConfig();
       if (bUseCustomGameMapLists)
           SpawnMapLists();                // only do this on server startup
       sMap = CustomGame[i].DefaultMap;
       if (sMap != "")
       {
           bRestarting = true;
           if (!bUseCustomGameMapLists)
               SetMapPath(CustomGame[i].MapFolder);
           else
               SetMapPath(CustomGame[i].MapFolder, true);
           SavePackageActors(i);
           SaveGameInfo(CustomGame[i].GameName);
           SaveServerName(i);
           sTemp = sMap$"?Game="$CustomGame[i].GameClass;
           Log("Server Startup - Restarting Default Game: "$sTemp, 'MapVoteX');
           bLevelSwitchPending = True;
           if (Level.NetMode == NM_Standalone)
               Level.ServerTravel(sTemp, False);
           else
           {
              ServerTravelString = sTemp;
              ServerTravelTime = Level.TimeSeconds + 1;
              SetTimer(1.0, true);
           }
           return false;
       }
       Log("#### MapVoteX: Cannot restart the Default Game: "$CustomGame[i].GameName$" as it's DefaultMap: "$sMap$" cannot be found!");
       HandleFailedMap(sMap);
       return true;
    }

	if ( DefaultSettings != "" )
	{
		Settings = DefaultSettings;
		if (Right(Settings, 1) != ",")
		    Settings = Settings$",";
		if ( Len(Settings) > 5 )
		{
 		   for (j=0; j<50; j++)
		   {
				pos = InStr(Settings,",");
				if ( pos < 0 )
				{
					break;
				}
				else
				{
					ExecuteSetting(Left(Settings, pos),True);
					Settings = Mid(Settings,pos + 1);
				}
			}
		}
	}

    if ( DefaultMutators != "" )
    {
       Mutators = DefaultMutators;
       if (Right(Mutators, 1) != ",")
           Mutators = Mutators$",";
       if ( Len(Mutators) > 3 )
       {
          for (j=0; j<50; j++)
          {
              pos = InStr(Mutators, ",");
              if ( pos < 0 )
                  break;
              else
              {
                 StartActorMutator(Left(Mutators, pos));
                 Mutators = Mid(Mutators,pos + 1);
              }
          }
       }
    }

    bFound = false;
    Mutators = "";
    Settings = "";

    for (i=0; i<CustomList; i++)
    {
        DebugLog("Searching for Game: "$MVGameName$" and found: "$CustomGame[i].GameName);
		if ( CustomGame[i].bUse && CustomGame[i].GameName == MVGameName)
		{
            bFound = true;
			if ( CustomGame[i].Mutators != "" )
			{
				Mutators = CustomGame[i].Mutators;
                if (Right(Mutators, 1) != ",")
                    Mutators = Mutators$",";
                if ( Len(Mutators) > 2 )
                {
                   for (j=0; j<50; j++)
                   {
                       pos = InStr(Mutators, ",");
                       if ( pos < 0 )
                       {
                          break;
                       }
                       else
                       {
                          StartActorMutator(Left(Mutators, pos));
                          Mutators = Mid(Mutators,pos + 1);
                       }
                   }
                }
			}

			if ( CustomGame[i].Settings != "" )
			{
				Settings = CustomGame[i].Settings;
				if (Right(Settings, 1) != ",")
				    Settings = Settings$",";
				for (j=0; j<100; j++)          // not likely to be more than 100
				{
					pos = InStr(Settings,",");
					if ( pos < 0 )
					{
						break;
					}
					else
					{
						ExecuteSetting(Left(Settings, pos),False);
						Settings = Mid(Settings,pos + 1);
					}
				}
			}
			if (bFound)
			   break;
		}
	}
	return true;
}

function StartActorMutator (string ClassString)
{
	local Actor A;
	local Class<Actor> ActorClass;

	Log("StartMutator:" @ ClassString,'MapVoteX');
	ActorClass=Class<Actor>(DynamicLoadObject(ClassString,Class'Class'));
	if ( ActorClass == None )
	{
		Log("Starting Actor/Mutator FAILED. Could not DynamicLoadObject:" @ ClassString,'MapVoteX');
		return;
	}
	foreach AllActors(ActorClass,A)
	{
		Log("Actor already spawned:" @ ClassString,'MapVoteX');
		return;
	}
	A=Spawn(ActorClass);
	if ( A == None )
	{
		Log("Spawning Actor/Mutator FAILED:" @ ClassString,'MapVoteX');
		return;
	}
	if ( A.IsA('Mutator') )
	{
		Level.Game.BaseMutator.AddMutator(Mutator(A));
	}
}

function ExecuteSetting (string Setting, bool bIsDefaultSetting)
{
	local string Property;
	local string Value;
	local string Prev;
	local string Next;

	Property = Left(Setting,InStr(Setting, "=" ));
	Value = Mid(Setting,InStr(Setting, "=") + 1);

	if ( bIsDefaultSetting )
		Log("Execute default Setting:" @ Setting $ ". Property:" @ Property $ ". Value:" @ Value,'MapVoteX');
	else
		Log("Execute Setting:" @ Setting $ ". Property:" @ Property $ ". Value:" @ Value,'MapVoteX');

	Prev = Level.Game.GetPropertyText(Property);
	Level.Game.SetPropertyText(Property,Value);
	Next = Level.Game.GetPropertyText(Property);
	DebugLog("Changed Setting:" @ Property $ ". Previous value:" @ Prev $ ". new value:" @ Next);
}

function string GetAbsoluteTime()
{
	local string AbsoluteTime;

	if (Level.Hour < 10)
		AbsoluteTime = AbsoluteTime$"0"$Level.Hour;
	else
		AbsoluteTime = AbsoluteTime$Level.Hour;

	if (Level.Minute < 10)
		AbsoluteTime = AbsoluteTime$":0"$Level.Minute;
	else
		AbsoluteTime = AbsoluteTime$":"$Level.Minute;

	if (Level.Second < 10)
		AbsoluteTime = AbsoluteTime$":0"$Level.Second;
	else
		AbsoluteTime = AbsoluteTime$":"$Level.Second;

	return AbsoluteTime;
}

function SaveGameInfo(string sGame)
{
    MVGameName = sGame;
    SaveConfig();
}

function SaveServername(int iGame)
{
     if (bMaintenanceMode)
         return;

     if (MVSN == None)
         MVSN  = Spawn(class'MapVoteServerNames');
     MVSN.MyMut = self;
     MVSN.DoServerName(iGame);
}

function SavePackageActors(int iGame)
{
    if (bEnableServerPackages)
    {
        MVSP = Spawn(class'MapVoteServerPackages');
        MVSP.MyMut = self;
        MVSP.WriteServerPackages(iGame);

        MVSA = Spawn(class'MapVoteServerActors');
        MVSA.MyMut = self;
        MVSA.WriteServerActors(iGame);
    }
}
/*
MapVoteXD.ini: Serverpackages and ServerActors
MapVoteXL.ini: Intro Logos and Welcome Logos
MapVoteXM.ini: Admin Messages
MapVoteX_MapLists: All the 30 maplists as required if required.
All ServerName stuff in MapVoteX.ini
*/
function SpawnAdvancedClasses(PlayerPawn Sender)
{
    if (CheckForAdmin(Sender) && !bClassesSpawned)
    {
        MVSP = Spawn(class'MapVoteServerPackages');
        MVSA = Spawn(class'MapVoteServerActors');
        MVCI = Spawn(class'MapVoteClientIntro');
        MVCI.GameNum = LoadedGameNum + 1;
        MVCI.bEnableLogo = false;
        MVCW = Spawn(class'MapVoteClientWelcome');
        MVAM = Spawn(class'MapVoteAdminMessages');
        MVSN = Spawn(class'MapVoteServerNames');
        MVCM = Spawn(class'MapVoteCrashedMaps');
        SpawnMapLists();
        bClassesSpawned = true;
    }
    Sender.ClientMessage("All Advanced File Classes have been Respawned!");
}

// called at server start only
function SpawnMapLists()
{
    Spawn(class'MVXCustomGame_00_Maplist');
    Spawn(class'MVXCustomGame_01_Maplist');
    Spawn(class'MVXCustomGame_02_Maplist');
    Spawn(class'MVXCustomGame_03_Maplist');
    Spawn(class'MVXCustomGame_04_Maplist');
    Spawn(class'MVXCustomGame_05_Maplist');
    Spawn(class'MVXCustomGame_06_Maplist');
    Spawn(class'MVXCustomGame_07_Maplist');
    Spawn(class'MVXCustomGame_08_Maplist');
    Spawn(class'MVXCustomGame_09_Maplist');
    Spawn(class'MVXCustomGame_10_Maplist');
    Spawn(class'MVXCustomGame_11_Maplist');
    Spawn(class'MVXCustomGame_12_Maplist');
    Spawn(class'MVXCustomGame_13_Maplist');
    Spawn(class'MVXCustomGame_14_Maplist');
    Spawn(class'MVXCustomGame_15_Maplist');
    Spawn(class'MVXCustomGame_16_Maplist');
    Spawn(class'MVXCustomGame_17_Maplist');
    Spawn(class'MVXCustomGame_18_Maplist');
    Spawn(class'MVXCustomGame_19_Maplist');
    Spawn(class'MVXCustomGame_20_Maplist');
    Spawn(class'MVXCustomGame_21_Maplist');
    Spawn(class'MVXCustomGame_22_Maplist');
    Spawn(class'MVXCustomGame_23_Maplist');
    Spawn(class'MVXCustomGame_24_Maplist');
    Spawn(class'MVXCustomGame_25_Maplist');
    Spawn(class'MVXCustomGame_26_Maplist');
    Spawn(class'MVXCustomGame_27_Maplist');
    Spawn(class'MVXCustomGame_28_Maplist');
    Spawn(class'MVXCustomGame_29_Maplist');
}

//
defaultproperties
{
    Version="1.0.3"
    bAutoOpen=True
    bAllowMidGameVoting=True
    bUseCommandOptions=True

    bEnableSplashLogo=True
    bEnableWelcomeMenu=True
    bEnableEmptyServerSwitch=True

    EmptyServerSwitchTimeMins=15
    VoteTimeLimit=50
    ScoreBoardDelay=10
    RepeatLimit=5
    GameStartVoteDelay=15
    DefaultCustomGameNum=0
    DefaultMutators=""
    DefaultSettings=""
    bSortCustomGameList=True
    CustomGame(0)=(bUse=True,bMapList=False,DefaultMap="DM-Deck16][",GameName="Tournament DeathMatch",GameClass="Botpack.DeathMatchPlus",Mutators="",Settings="")
    CustomGame(1)=(bUse=True,bMapList=False,DefaultMap="DM-Deck16][",GameName="Tournament Team Game",GameClass="Botpack.TeamGamePlus",Mutators="",Settings="")
    CustomGame(2)=(bUse=True,bMapList=False,DefaultMap="CTF-Face",GameName="Capture the Flag",GameClass="Botpack.CTFGame",Mutators="",Settings="")
    MVGameName="Tournament DeathMatch"
}

